/**
 * Modal 0.0.1
 * A portable modal module
 * @author Kyle Foster (@hkfoster)
 * @license MIT
 **/

// Dependencies
var toolbox    = require( 'compose-toolbox' ),
    Event      = toolbox.event,
    matches    = toolbox.matches,
    getClosest = toolbox.getClosest,
    once       = require( '../utility/once' );

function getTriggerPosition( event ) {
  var left   = event.clientX,
      top    = event.clientY,
      right  = window.innerWidth - left,
      bottom = window.innerHeight - top;

  return {
    left   : left,
    right  : right,
    top    : top,
    bottom : bottom
  }
}

function focusInput( parent, order ) {
  order = order - 1 || 0;
  if ( parent.querySelectorAll( 'input' ).length ) parent.querySelectorAll( 'input' )[ order ].focus();
}

function pythagorean( sideA, sideB ){
  return Math.sqrt( Math.pow( sideA, 2 ) + Math.pow( sideB, 2 ) );
}

// Public API function
var modal = function( selector, content, callback ) {

  // Click handler function
  var clickHandler = function( event ) {

    // If `close-modal` button is clicked
    if ( getClosest( event.target, '[data-close-modal]' ) ) {

      // Get outta Dodge
      closeModal();

    // If attached selector is clicked
    } else if ( getClosest( event.target, selector ) ) {

      // Show the modal
      showModal( event );

    }

    // Otherwise just bail
    return false;

  };

  // Show modal function
  var showModal = function( event ) {

    // Prevent default behavior
    event.preventDefault();

    // Prevent page scroll
    document.body.style.overflow = 'hidden';

    // If modal has yet to be initialized, insert it into the document
    if ( !document.querySelector( '[data-modal]' ) ) document.body.insertAdjacentHTML( 'beforeend', '<aside data-modal><section data-modal-content></section><a data-close-modal><svg viewBox="0 0 30 30" width="16"><path d="M2 2l26 26M2 28l26-26" stroke="currentColor" stroke-width="4" stroke-linecap="round" fill="none" /></svg><span>esc</span></a></aside>' );

    // Assignment
    var
    modalTrigger  = event.target,
    modalElement  = document.querySelector( '[data-modal]' ),
    modalContent  = modalElement.querySelector( '[data-modal-content]' ),
    modalBackdrop = modalElement.querySelector( '[data-modal-backdrop]' ),

    // Calculate trigger's position in the viewport
    position = getTriggerPosition( event ),

    // Calculate the circle's radius
    radius = pythagorean( window.innerHeight, window.innerWidth ),

    // Assign position coordinates
    xPosition = position.left - Math.round( radius ),
    yPosition = position.top - Math.round( radius ),

    // Position modal background accordingly
    modalStyles = '<style data-modal-coords>[data-modal]:after { ' +
      'left: ' + xPosition + 'px; ' +
      'top: ' + yPosition + 'px; ' +
      'width: ' + radius * 2 + 'px; ' +
      'height: ' + radius * 2 + 'px; ' +
    '}</style>';

    // If content is passed
    if ( content ) {

      // And it’s a function
      if ( content instanceof Function ) {

        // Make it our callback
        callback = content;

      // Otherwise
      } else {

        // Set modal content accordingly
        modalContent.innerHTML = content;

      }

    }

    // Insert modal positioning styles into document
    document.body.insertAdjacentHTML( 'beforeend', modalStyles );

    // Wait for that sumbitch to animate
    once.evt( modalElement, 'animationend', function( event ) {

      // Then make its contents visible
      modalElement.setAttribute( 'data-modal', 'visible' );

      // Focus first input
      focusInput( modalElement );

    }, false );

    // Activate modal
    modalElement.setAttribute( 'data-modal', 'in' );

    // Run callback function
    if ( callback ) once.fn( callback( event ) );

  }

  var keyHandler = function( event ) {
    event = event || window.event;
    var isEscape;

    if ( 'key' in event ) {
      isEscape = ( event.key == 'Escape' || event.key == 'Esc' );
    } else {
      isEscape = ( event.keyCode == 27 );
    }

    if ( isEscape ) {
      closeModal();
    }
  }

  var closeModal = function( event ) {
    var activeModal = document.querySelector( '[data-modal="visible"]' );

    if ( !activeModal ) return false;

    activeModal.setAttribute( 'data-modal', 'out' );

    once.evt( activeModal, 'animationend', function( event ) {

      activeModal.setAttribute( 'data-modal', '' );

      activeModal.querySelector( '[data-modal-content]' ).innerHTML = '';

      document.body.style.overflow = '';

      var styles = document.querySelector( '[data-modal-coords]' );

      if ( styles ) document.body.removeChild( styles );

    }, false );

  }

  document.addEventListener( 'click', clickHandler, false );
  document.addEventListener( 'keydown', keyHandler, false );

}

// Public API
module.exports = modal;