/**
 * Waypoints 0.0.4
 * Compose waypoint navigation module
 * @author Kyle Foster (@hkfoster)
 * @license MIT
 **/

// Dependencies
var toolbox    = require( 'compose-toolbox' ),
    Event      = toolbox.event,
    matches    = toolbox.matches,
    fromTop    = toolbox.fromTop,
    scrollTo   = toolbox.scrollTo,
    getClosest = toolbox.getClosest,
    callback   = Event.callback;

// Public API function
var waypoints = function( settings ) {

  // Overridable defaults
  var defaults = {
    initWidth        : '0px',
    elemOffset       : 0
  }

  function slice( obj, count ) {
    return Array.prototype.slice.call( obj, count )
  }

  // Scoped variables
  var options          = toolbox.merge( defaults, settings ),
      activeAnchor     = 'active',
      landmarkSelector = 'dl',
      activeLandmark   = 'pinned',
      nav              = document.querySelector('.waypoint-nav'),
      elements         = [];

  if ( nav ) slice( nav.querySelectorAll( "a[href^='#']" ) ).forEach( function( el ) {

    // Find all elements matching links in the waypoints nav
    var section = document.querySelector( el.getAttribute('href') )

    // If found, add them to the elements list
    if ( section ) elements.push( section )

  })

  if ( !nav || elements.length < 1 ) { return false }

  // Scoped variables
  var docBody     = document.body,
      coordinates = [],
      oldActiveItem,
      windowHash,
      docHeight,
      winHeight;

  // If hash is present
  if ( window.location.hash ) {

    // Cache it
    windowHash = window.location.hash.replace( '#', '' );

    // Then delete it to prevent default page scroll
    window.location.hash = '';

    // And replace it
    history.replaceState( null, '', '#' + windowHash );

  }

  // Only run once the window is loaded
  Event.on( window, 'load', function() {

    // Media query handler function
    Event.media.minWidth( options.initWidth, function( condition ) {

      // Set up resize listener
      resize.toggle( condition.matches )
      resize()

      // Set up scroll listener
      scroll.toggle( condition.matches )
      scroll()
      
      // Set up hash change listener
      hashChange.toggle( condition.matches )
      hashChange()

    })

  })

  // Resize handler function
  var resize = callback.new( function() {

    // Update document height variable
    docHeight = docBody.scrollHeight;

    // Update window height variable
    winHeight = window.innerHeight;

    // Loop through waypoints
    setCoordinates()

  })

  // Store coordinates to elements
  var setCoordinates = function() {
    elements.forEach( function( element, i ) {
      coordinates[ i ] = {
        elem   : elements[ i ],
        offset : fromTop( elements[ i ] )
      }
    })
  }

  // Scroll handler function
  var scroll = callback.new( function() {

    // Scoped variables
    var newScrollY = window.pageYOffset,
        rockBottom = newScrollY < 0 || newScrollY + winHeight >= docHeight,
        newActiveItem,
        activeLink,
        landmark;

    // Loop through coordinates
    coordinates.forEach( function( currWaypoint, i ) {

      // Scoped variables
      var firstWaypoint = coordinates[ 0 ],
          nextWaypoint  = coordinates[ i + 1 ],
          lastWaypoint  = coordinates[ coordinates.length - 1 ];

      // If scrolled to last waypoint
      if ( newScrollY < firstWaypoint.offset ) {

        // Set active item to first waypoint
        newActiveItem = firstWaypoint.elem.id;

      // If scrolled to bottom of page or on past the last waypoint
      } else if ( rockBottom || newScrollY > lastWaypoint.offset - options.elemOffset ) {

        // Set active item to last waypoint
        newActiveItem = lastWaypoint.elem.id;

      // Otherwise if a waypoint is in the viewport
      } else if ( newScrollY > currWaypoint.offset - options.elemOffset && newScrollY < nextWaypoint.offset - options.elemOffset ) {

        // Update active item to match
        newActiveItem = currWaypoint.elem.id;

      }

    })

    // If active item exists
    if ( newActiveItem ) {

      // Only run when new active item is triggered
      if ( newActiveItem === oldActiveItem ) return false

      // Update active link
      activeLink = nav.querySelector( 'a[href="#' + newActiveItem + '"]' )

      // If no active link is found, abort
      if ( !activeLink ) return false

      // And enable navigation item
      activateLink( activeLink )

      // Update old active item variable
      oldActiveItem = newActiveItem

    }

  })

  // Click handler function
  var click = callback.new( function( event ) {

    // Matches selector function init
    if ( matches( event.target, 'a[href^="#"]' ) ) {

      // Prevent default behavior
      event.preventDefault()

      // Travel to clicked target
      travel( event.target, true )

    }
  })

  // Hash change handler function
  var hashChange = callback.new( function() {

    // Only run when a hash is present
    if ( !window.location.hash ) return false

    // Find the hash’s relevant navigation item
    var hashAnchor = document.querySelector( 'a[href="' + window.location.hash + '"]' )

    // And travel to its target
    travel( hashAnchor )

  })

  // Window scroll travel function
  var travel = function( destination, animateScroll ) {

    // Define scoped variable(s)
    var targetAnchor   = destination.getAttribute('href'),
        targetElement  = document.querySelector( targetAnchor ),
        targetOffset,
        travelTime;

    // If a target element exists
    if ( targetElement ) {

      // Enable new active navigation item
      activateLink( destination, activeAnchor )

      // Loop through coordinates
      coordinates.forEach( function( current, i ) {

        // If coordinate element matches target element
        if ( coordinates[ i ].elem === targetElement ) {

          // Assign its coordinates to target offset variable
          targetOffset = coordinates[ i ].offset;
        }
      })

      // If trigger was a click
      if ( animateScroll ) {

        // Temporarily remove scroll listener function
        scroll.stop()

        // Update the hash
        history.pushState( null, '', targetAnchor );

        // Animate scroll to appropriate element then renable scroll listener
        scrollTo( targetElement, scroll.start )

      } else {

        // Skip the scroll animation
        window.scrollTo( 0, fromTop( targetElement ) );

      }
    }
  }

  function activate( el, classname ) {

    // Find currently active link
    var activeEl = nav.querySelector( '.' + classname )

    if ( activeEl == el ) return

    // remove active class from existing element
    if ( activeEl ) activeEl.classList.remove( classname ) 

    // Then enable the passed selector
    el.classList.add( classname )

  }

  // Link activation function
  function activateLink( el ) {
    activate( el, activeAnchor )

    // Set active landmark to active link’s parent
    activateLandmark( getClosest( el, landmarkSelector ) )
  }

  function activateLandmark( el ) {
    if ( el ) activate( el, activeLandmark )
  }

  Event.resize.stop( resize )
  Event.scroll( scroll )
  Event.on( document, 'click', click )
  Event.on( window, 'hashchange', hashChange )

};

// Public API
module.exports = waypoints;
