module Ratchet
  module PricingHelper

    TYPE_SIZE = {
      'data'    => { ram: 0.1, storage: 1 },
      'memory'  => { ram: 0.25, storage: 0.25 },
      'memdata' => { ram: 0.25, storage: 0.25 }
    }

    steps = [1, 2, 5, 10, 25, 100]
    ranges = [[1,10],[12,22],[25,50],[60,120],[125,350],[400,1000]]
    units = []

    ranges.each_with_index do |range, index|
      (range.first .. range.last).step(steps[index]) do |step|
         units.push step
      end
    end

    UNITS = units

    def pricing_data
      @deployment_pricing ||= begin
        deployment_data = Site.config_data['deployments']

        # Only select pricing for deployments we have planned UI for
        pricing = Site.config_data['pricing'].select { |d| deployment_data['slugs'][ d['slug'] ] }

        deployment_pricing = {}
        pricing.each do |deployment|

          slug       = deployment_data['slugs'][ deployment['slug'] ]
          overhead   = deployment['base_overhead'] || 0
          type       = deployment['scalables'].first['type']
          type_size  = TYPE_SIZE[type] # Storage/Memory configuration for this deployment type
          tiers      = deployment['scalables'].first['price_tiers']
          unit_cost  = tiers.first['unit_cost'] # Get per unit price with no discount
          max_storage = deployment_data['deployments'][slug][:max_storage_units] || UNITS.last
          max_ram = deployment_data['deployments'][slug][:max_ram_units] || UNITS.last

          new_data   = {
            type:        type,
            units:       [],
            price:       [],
            price_overhead:  overhead,
            price_unit:  unit_cost,
            price_ph:    [],
            discount:    [],
            savings:     [],
            storage:     [],
            memory:      []
          }
          
          units = UNITS.select {|u| tiers.first['range'][0] <= u }

          tiers.each do |tier|

            range     = units.select{ |u| tier['range'].first <= u && u <= tier['range'].last }
            cost      = tier['unit_cost']
            discount  = tier['discount_pct']

            new_data[:units].concat     range
            new_data[:price].concat     range.map{|r| number_to_currency((cost * r) + overhead) }
            new_data[:price_ph].concat  range.map{|r| (((cost * r) + overhead).to_f / 720).round(4) }
            new_data[:discount].concat  range.map{|r| discount }
            new_data[:savings].concat   range.map{|r| number_to_currency(((cost * r) + overhead) * discount/100 )}
            new_data[:storage].concat   range.map{|r| (r < max_storage ? r : max_storage) * type_size[:storage] }
            new_data[:memory].concat    range.map{|r| (r < max_ram ? r : max_ram) * type_size[:ram] }

          end

          deployment_pricing[slug] = new_data

        end

        deployment_pricing
      end
    end

    def pricing_overview(slug)
      if data = pricing_data[slug]
        unit_size = data[:type] == 'data' ? data[:storage] : data[:memory]

        {
          starting_price: '$' + number_to_currency(data[:price].first).sub(/\.00$/,''),
          unit_price: '$' + number_to_currency(data[:price_unit]).sub(/\.00$/,''),
          unit_size: nearest_size(unit_size[2] - unit_size[1]),
          unit_type: (data[:type] == 'data' ? 'Storage' : 'RAM')
        }
      end
    end

    def number_to_currency(value)
      ActionController::Base.helpers.number_to_currency(value, unit: '')
    end

    def pricing_slider(name, options={})
      data = pricing_data[name].merge(options)


      data[:mark] = []
      data[:storage].map!{ |s| nearest_size(s) }
      data[:memory].map!{ |s| nearest_size(s) }

      label_with = data[:type] == 'data' ? data[:storage] : data[:memory]

      data[:line_labels] = { 
        "1" => label_with.first,
        "#{data[:units].size}" => label_with.last 
      }

      current_discount = 0

      data[:discount].each_with_index do |d, index|
        if d != current_discount
          current_discount = d
          data[:mark].push index + 1
        end
      end

      data[:values] = data.delete(:units)

      data[:external_labels] = {
        discount: data.delete(:discount).map{|d| d == 0 ? '' : d },
        savings:  data.delete(:savings).map{|d| d == '0.00' ? '' : d },
        storage:  data.delete(:storage),
        memory:   data.delete(:memory),
        price_m:  data.delete(:price),
        price_h:  data.delete(:price_ph)
      }

      data

    end

    def nearest_size(size)
      if size > 1024
        "#{(size / 1024).round(2)} TB"
      else
        size = (size * 1024 * 1048576).round(2) # GiB -> MiB -> Bytes
        ActionController::Base.helpers.number_to_human_size(size, precision: 3).sub(/\.0 /,' ')
      end
    end
  end
end
